package main;

import main.Visualizer.Layer;

/**
 * A hangyabolyt modellezi. Az idő múlásával folyamatosan újabb hangyákat hoz
 * létre.
 * 
 * @author Kozaroczy
 */
public class AntHill extends NonBlocking implements ITimed {

	/** Ezzel számolja, hogy jöhet-e új hangya */
	protected int unusedTicks;
	/** Ezzel számolja, hogy mennyi időnként kell jönnie hangyának */
	protected int tickPerAnt = 100;

	/**
	 * Hangyaboly konstruktora, beregisztrálja magát a {@link Director}hoz
	 * 
	 * @param location
	 *            A hangyaboly helye
	 */
	public AntHill(HexCell location) {
		super(location);
		Logger.entering(this, new Object[] { location });
		Director.getInstance().registerTimed(this);
		Director.getInstance().setAntHillLocation(location);
		unusedTicks = 0;
		Logger.exiting(this);
	}

	/**
	 * Meghívja az {@link IInterActor#interactWith(AntHill)} függvényét.
	 * 
	 * @param interactor
	 *            Interaktor, aki reakciót kér
	 */
	public void accept(IInterActor interactor) {
		Logger.entering(this, new Object[] { interactor });
		interactor.interactWith(this);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void dispose() {
		Logger.entering(this, null);
		Director.getInstance().deregisterTimed(this);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public String draw() {
		return "img/anthill.png";
	}

	/**
	 * A hangya ezen keresztül jelzi, hogy behozott egy ételt.
	 * 
	 * @see Director#foodCollected()
	 * 
	 */
	public void foodCollected() {
		Logger.entering(this, null);
		Director.getInstance().foodCollected();
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public Layer getLayer() {
		return Layer.OBSTACLE;
	}

	/**
	 * {@inheritDoc} Megfelelő időközönként új hangyát helyez el a pályára
	 * 
	 */
	public void tick() {
		Logger.entering(this, null);
		if (Main.RND) {
			/** Véletlenszerűen rak le új hangyát, maximum tickPerAnt-onként */
			int rnd = (int) (Math.random() * tickPerAnt);
			if (unusedTicks >= rnd) {
				/** Ha kell, akkor lerak egyet */
				unusedTicks = 0;
				new Ant(location);
				Logger.exiting(this);
				return;
			} else {
				unusedTicks++;
			}
			/**
			 * Véletlenszerűen csökkenti a tickPerant értékét, így egyre
			 * gyakrabb jön majd hangya
			 */
			rnd = (int) Math.random() * 200;
			tickPerAnt = rnd < 8 ? tickPerAnt - 1 : tickPerAnt;
			tickPerAnt = tickPerAnt < 5 ? 5 : tickPerAnt;
		} else {
			new Ant(location);
		}
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "anthill";
	}

}
