package main;

import main.Visualizer.Layer;

/**
 * Hangyaszagot reprezentáló osztály
 * 
 */
public class AntScent extends Scent {

	/**
	 * Hangyák befolyásolásának mértéke.
	 * 
	 * @see Ant#move(HexCell)
	 */
	protected int influence = 2;
	
	/**
	 * Alapértelmezett konstruktor
	 * 
	 * @param location
	 *            Kezdőpozíció
	 */
	public AntScent(HexCell location) {
		super(location, 0, 6);
		Logger.entering(this, new Object[] { location });
		Logger.exiting(this);

	}

	/**
	 * Részletes konstruktor a betöltéshez
	 * 
	 * @param location
	 *            A szagnyom helye
	 * @param remaining
	 *            A maradék idő
	 */
	public AntScent(HexCell location, int remaining) {
		super(location, 0, remaining);
		Logger.entering(this, new Object[] { location, remaining });
		Logger.exiting(this);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void accept(IInterActor interactor) {
		Logger.entering(this, new Object[] { interactor });
		interactor.interactWith(this);
		Logger.exiting(this, null);
	}

	/**
	 * {@inheritDoc}
	 */
	public String draw() {
		return "img/antscent.png";
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Layer getLayer() {
		return Layer.SCENT;
	}

	/**
	 * A hangyák befolyásolása. Függ a befolyásoló képességtől és a szag
	 * intenzitásától
	 * 
	 * @see #influence
	 * @see Scent#remaining
	 */
	public int influence() {
		return influence * remaining;
	}

	/**
	 * {@inheritDoc} Nem tud terjedni.
	 * 
	 */
	public void tick() {
		Logger.entering(this, null);
		remaining--;
		if (remaining == 0) {
			dispose();
		}
		Logger.exiting(this, null);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "antscent " + remaining;
	}

}
