package main;

import java.util.ArrayList;
import java.util.HashMap;

import main.HexCell.Direction;
import main.Visualizer.Layer;

/**
 * Hangyaszagsemlegesítő szagot reprezentáló osztály
 * 
 */
public class AntScentVanisherScent extends Scent implements IInterActor {

	/**
	 * Alapértelmezett konstruktor
	 * 
	 * @param location
	 *            A szagnyom helye
	 * @param size
	 *            A terjedés mértéke
	 */
	public AntScentVanisherScent(HexCell location, int size) {
		super(location, size, 3);
		Logger.entering(this, new Object[] { location });
		Logger.exiting(this);
	}

	/**
	 * Terjedéshez használt konstruktor
	 * 
	 * @param location
	 *            Az új szagnyom helye
	 * @param size
	 *            A terjedés mértéke
	 * @param dir
	 *            A továbbterjedés iránya
	 */
	public AntScentVanisherScent(HexCell location, int size, Direction dir) {
		super(location, size, 1, dir);
		Logger.entering(this, new Object[] { location });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void accept(IInterActor interactor) {
		Logger.entering(this, new Object[] { interactor });
		interactor.interactWith(this);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public String draw() {
		return "img/antscentvanisherscent.png";
	}

	/**
	 * {@inheritDoc}
	 */
	public Layer getLayer() {
		return Layer.SCENT;
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(Ant a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(AntHill a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(Antlion a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc} Megsemmisíti azt
	 * 
	 * @param a
	 *            Hangyaszag
	 * 
	 */
	public void interactWith(AntScent a) {
		Logger.entering(this, new Object[] { a });
		a.dispose();
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(AntScentVanisherScent a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(Echidna e) {
		Logger.entering(this, new Object[] { e });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(Food f) {
		Logger.entering(this, new Object[] { f });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(FoodScent s) {
		Logger.entering(this, new Object[] { s });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(Pebble p) {
		Logger.entering(this, new Object[] { p });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(Poison p) {
		Logger.entering(this, new Object[] { p });
		Logger.exiting(this);
	}
	
	
	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(Puddle p) {
		Logger.entering(this, new Object[] { p });
		Logger.exiting(this);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Slime s) {
		Logger.entering(this, new Object[] { s });
		// Nothing to do here
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Snail s) {
		Logger.entering(this, new Object[] { s });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void tick() {
		Logger.entering(this, null);
		if (radius > 0) {
			HashMap<Direction, HexCell> adjacent = location.getAdjacents();
			if (first) {
				/** Ha ez az első, akkor minden irányba kell terjedni */
				for (Direction d : Direction.values()) {
					HexCell c = adjacent.get(d);
					if (c != null)
						new AntScentVanisherScent(c, radius - 1);
				}
			} else {
				/** Egyébként minden előírt irányba terjed, ha van ott cella */
				for (int i = 0; i < dir.size(); i++) {
					HexCell c = adjacent.get(dir.get(i));
					if (c != null) {
						new AntScentVanisherScent(c, radius - 1, dir.get(i));
					}
				}
			}
			/** Ha egyszer már terjedt, akkor ne terjedjen többször */
			radius = 0;
		}
		/** Reagál mindennel a saját tick-jében */
		ArrayList<Located> others = location.getAll();
		for (int i = others.size() - 1; i >= 0; i--) {
			others.get(i).accept(this);
		}
		/** Minden körben csökken az intenzitása */
		remaining--;
		if (remaining == 0) {
			dispose();
		}
		Logger.exiting(this);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "antscentvanisher " + radius;
	}
}
