package main;

import main.Visualizer.Layer;

/**
 * Hangyalesőket reprezentáló osztály
 * 
 * @author Kozaroczy
 *
 */
public class Antlion extends NonBlocking {

	/**
	 * Hangyaleső kontruktora.
	 * 
	 * @param location
	 *            Alapértelmezett pozíció
	 */
	public Antlion(HexCell location) {
		super(location);
		Logger.entering(this, new Object[] { location });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc} 
	 */
	public void accept(IInterActor interactor) {
		Logger.entering(this, new Object[] { interactor });
		interactor.interactWith(this);
		Logger.exiting(this);
	}


	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "antlion";
	}

	/**
	 * Jelzi, hogy egy olyan hangyát evett meg, amelyiknél volt étel, így az
	 * megsemmisült
	 * 
	 * @see Director#foodDestroyed()
	 * 
	 */
	public void foodDestroyed() {
		Logger.entering(this, null);
		Director.getInstance().foodDestroyed();
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public String draw() {
		return "img/antlion.png";
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Layer getLayer() {
		return Layer.OBSTACLE;
	}
}