package main;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.Set;

import main.HexCell.Direction;
import main.LoaderSaver.doSpraying;

/**
 * A játék inicializálásáért és futtatásáért felelő osztály.
 * 
 * @author pgy
 * @author Kozaroczy
 * @author totha
 * 
 */
public class Director implements ITimed {
	/** A singleton példány */
	private static Director theOne = null;

	/**
	 * Az aktuális példányt adja vissza. Ha még nem létezik, akor létrehoz egy
	 * példányt.
	 * 
	 * @return Az egyetlen példány
	 */
	public static Director getInstance() {
		Logger.entering(theOne, null);
		if (theOne == null) {
			try {
				theOne = new Director();
			} catch (SingletonException s) {
				Logger.meanwhiling("Jajj! Egy Director nem elég?");
				System.err.println("Singleton error");
			}
		}
		Logger.exiting(theOne, theOne);
		return theOne;
	}

	/** Az aktív spray */
	protected Spray activeSpray;
	/** Az összes cella */
	protected ArrayList<HexCell> cells;
	/** A maradék ételek száma */
	protected int foodRemaining;
	/** A használható spray-k */
	protected ArrayList<Spray> sprays;
	/** A betöltéskor meghatározott késleltetett fújásokhoz */
	protected ArrayList<doSpraying> sprayingList;
	/** A pálya mérete */
	protected int size_x, size_y;
	/** A hangyaboly helye */
	protected HexCell antHillLocation;
	/** A hangyák száma */
	protected int antCount = 0;
	/** A hangyászsünök száma */
	protected int echidnaCount = 0;
	/** Az összes időzített objektum */
	protected LinkedList<ITimed> timedObjects;
	/**
	 * Megadja, hogy hány hangyánként rendel be a pályára egy újabb hangyászsünt
	 */
	protected final int echidnaAttractingAntQuantity = 5;

	/**
	 * Konstruktor. Első híváskor beállítja a Director.theOne statikus mezőt egy
	 * új director példányra. Minden további hívásra kivételt dob. Inkább a
	 * {@link #getInstance()} függvény használata ajánlott.
	 * 
	 * @throws SingletonException
	 *             Ha már van egy Director példány.
	 * @see #getInstance()
	 */
	public Director() throws SingletonException {
		Logger.entering(this, null);
		if (theOne != null)
			throw new SingletonException();
		theOne = this;

		activeSpray = null;
		cells = new ArrayList<HexCell>();
		timedObjects = new LinkedList<ITimed>();
		sprays = new ArrayList<Spray>();
		sprayingList = new ArrayList<doSpraying>();
		Logger.exiting(this);
	}

	/**
	 * Hozzáad egy elemet a megfelelő cellához
	 * 
	 * @param x
	 *            Koordináta
	 * @param y
	 *            Koordináta
	 * @param l
	 *            Hozzáadandó
	 */
	public void add(int x, int y, Located l) {
		Logger.entering(this, new Object[] { x, y, l });
		cells.get(size_x * y + x).addObject(l);
		Logger.exiting(this);
	}

	/** Szagok összevonása */
	public void contractionOfScent() {
		Logger.entering(this, null);
		/** Végigmegy az összes cellán */
		for (HexCell c : cells) {
			ArrayList<Located> all = c.getAll();
			ArrayList<FoodScent> fs = new ArrayList<FoodScent>();
			
			/** Kigyűjti az ételszagokat */
			for (Located l : all) {
				if (l instanceof FoodScent) {
					fs.add((FoodScent) l);
				}
			}
			
			/** Ha egynél többet talál, van értelme összevonni */
			if (fs.size() > 1) {
				FoodScent theSurvival = fs.get(0);
				int max_rem = theSurvival.remaining;
				for (int i = 1; i < fs.size(); i++) {
					theSurvival.addOrigin(fs.get(i).origins);
					theSurvival.addDirs(fs.get(i).dir);
					max_rem = fs.get(i).remaining > max_rem ? fs.get(i).remaining : max_rem;
					fs.get(i).remaining = 0;
				}
				theSurvival.remaining = max_rem;
			}
		}
		Logger.exiting(this);
	}

	/**
	 * Csökkenti a hangyaszámlálót.
	 */
	public void decAnt() {
		Logger.entering(this, null);
		antCount--;
		Logger.exiting(this);
	}

	/**
	 * Csökkenti a hangyászsünszámlálót
	 */
	public void decEchidna() {
		Logger.entering(this, null);
		echidnaCount--;
		Logger.exiting(this);
	}

	/**
	 * Egy {@link ITimed} elem eltávolítása a listából
	 * 
	 * @param t
	 *            Eltávolítandó elem
	 */
	public void deregisterTimed(ITimed t) {
		Logger.entering(this, new Object[] { t });
		timedObjects.remove(t);
		Logger.exiting(this);
	}

	/**
	 * Az egyetlen példány megsemmisítése
	 * 
	 */
	public void dispose() {
		Logger.entering(this, null);
		theOne = null;
		Logger.exiting(this);
	}

	/**
	 * Az egyetlen példányt kinullázza
	 * 
	 */
	public void finalize() throws Throwable {
		Logger.entering(this, null);
		theOne = null;
		Logger.exiting(this);
	}

	/**
	 * Ezen keresztül kap jelzést, hogy egy étel beért a hangyabolyba. Csökkenti
	 * az ételszámlálót
	 * 
	 */
	public void foodCollected() {
		Logger.entering(this, null);
		foodRemaining--;
		Logger.exiting(this);
	}

	/**
	 * Ezen keresztül kap értesítést, hogy egy étel megsemmisült. Ekkor egy
	 * véletlenszerűen kiválasztott cellára rak egy új ételt. A véletlen cella
	 * csak olyan lehet, ami nem tartalmaz sem {@link Barrier}t, sem
	 * {@link Antlion}t, hiszen azok elérhetetlenek egy hangya számára. Továbbá
	 * {@link AntHill}t sem tartalmazhat.
	 */
	public void foodDestroyed() {
		Logger.entering(this, null);
		foodRemaining--;

		Random random = new Random();
		HexCell target = null;
		randomTarget: do {
			target = cells.get(random.nextInt(cells.size()));
			for (Located l : target.getAll()) {
				if (l instanceof Barrier || l instanceof Antlion || l instanceof AntHill) {
					continue randomTarget;
				}
			}
			break;
		} while (true);
		new Food(target);
		Logger.exiting(this);
	}

	/**
	 * Ezen keresztül kap jelzést, ha egy új étel jön létre
	 */
	public void foodGenerated() {
		Logger.entering(this, null);
		foodRemaining++;
		Logger.exiting(this);
	}

	/**
	 * Legenerálja az összes cellához a hangyabolyba vezető utat szélességi
	 * bejárással. Gyors.
	 */
	public void generatePaths() {
		Logger.entering(this, null);
		class Node {
			HexCell cell;
			LinkedList<HexCell> path;
			Direction backto;

			Node(HexCell cell, LinkedList<HexCell> path, Direction backto) {
				this.cell = cell;
				this.path = path;
				this.backto = backto;
			}
		}
		HexCell current = antHillLocation;
		HashSet<HexCell> processed = new HashSet<HexCell>();
		ArrayDeque<Node> rem = new ArrayDeque<Node>();
		rem.addLast(new Node(current, new LinkedList<HexCell>(), null));
		processed.add(current);
		while (!rem.isEmpty()) {
			Node n = rem.removeFirst();
			n.cell.setPath(n.path, n.backto);
			@SuppressWarnings("unchecked")
			LinkedList<HexCell> newPath = (LinkedList<HexCell>) n.path.clone();
			newPath.addFirst(n.cell);
			for (Direction d : Direction.values()) {
				Direction b = d.back();
				HexCell next = n.cell.getCell(d);
				if (next != null && !processed.contains(next)) {
					boolean pebble = false;
					for (Located l : next.getAll()) {
						if (l instanceof Pebble) {
							pebble = true;
						}
					}
					processed.add(next);
					if (!pebble) {
						rem.add(new Node(next, newPath, b));
					}
				}
			}
		}
		Logger.exiting(this);
	}

	/**
	 * Visszaadja, hol van a hangyaboly
	 * 
	 * @return A hangyaboly helye
	 */
	public HexCell getAntHillLocation() {
		return antHillLocation;
	}

	/**
	 * Koornidáták alapján visszaad egy HexCellt. 0-tól indexel.
	 * 
	 * @param x
	 *            X koordináta.
	 * @param y
	 *            Y koordináta.
	 * @return Az [x,y] koordinátájú HexCell.
	 * @throws ArrayIndexOutOfBoundsException
	 *             Ha negatív vagy túl nagy koordinátákat adtunk meg.
	 */
	public HexCell getCell(int x, int y) {
		if (x < 0 || x >= size_x || y < 0 || y >= size_y)
			throw new ArrayIndexOutOfBoundsException("Bad coordinates [" + x + "," + y + "] on map of [" + size_x + "," + size_y + "]");
		if (y * size_x + x > cells.size()) {
			System.err.println("Uninitialized array: Director::cells");
			setSize(size_x, size_y);
		}
		return cells.get(y * size_x + x);
	}

	/**
	 * Visszaadja az összes cellát
	 * 
	 * @return Az összes cella
	 */
	public ArrayList<HexCell> getCells() {
		Logger.entering(this, null);
		Logger.exiting(this, cells);
		return cells;
	}

	/**
	 * Az aktív spay-t adja vissza.
	 * 
	 * @return Az aktív Spray
	 * 
	 */
	public Spray getSpray() {
		Logger.entering(this, null);
		Logger.exiting(this, activeSpray);
		return activeSpray;
	}

	/**
	 * Növeli a hangyaszámlálót. A hangya hívja meg a konstruktorában
	 */
	public void incAnt() {
		Logger.entering(this, null);
		antCount++;
		Logger.exiting(this);
	}

	/**
	 * Növeli a hangyászsünszámlálót
	 */
	public void incEchidna() {
		Logger.entering(this, null);
		echidnaCount++;
		Logger.exiting(this);
	}

	/**
	 * A játék inicializálása.
	 * 
	 */
	public void initialize() {
		Logger.entering(this, null);

		/** Változók nullázása */
		foodRemaining = 0;
		antHillLocation = null;
		antCount = 0;
		echidnaCount = 0;
		activeSpray = null;
		timedObjects = new LinkedList<ITimed>();
		sprays = new ArrayList<Spray>();

		HashSet<HexCell> blocked = new HashSet<HexCell>();
		/** AntHill */
		HexCell antHill = randomCell();
		new AntHill(antHill);
		blocked.add(antHill);
		/** Pebble */
		int pebbleCount = randomInRange(size_x, (size_x * size_y) / 4);
		outer: for (int i = 0; i < pebbleCount; i++) {
			HexCell mycell;
			int tries = 0;
			do {
				mycell = randomCell();
				tries++;
				if (tries > size_x * size_y)
					break outer;
			} while (blocked.contains(mycell));
			new Pebble(mycell);
			blocked.add(mycell);
		}
		/** Antlion(s) */
		int antlionCount = randomInRange(size_x / 4, (size_x * size_y) / 16);
		outer: for (int i = 0; i < antlionCount; i++) {
			HexCell mycell;
			int tries = 0;
			do {
				mycell = randomCell();
				tries++;
				if (tries > size_x * size_y)
					break outer;
			} while (blocked.contains(mycell));
			new Antlion(mycell);
			blocked.add(mycell);
		}
		/** Puddle(s) */
		int puddleCount = randomInRange(size_x / 2, (size_x * size_y) / 8);
		outer: for (int i = 0; i < puddleCount; i++) {
			HexCell mycell;
			int tries = 0;
			do {
				mycell = randomCell();
				tries++;
				if (tries > size_x * size_y)
					break outer;
			} while (blocked.contains(mycell));
			new Puddle(mycell);
		}
		/** Food */
		int foodCount = randomInRange(size_x / 2, (size_x * size_y) / 8);
		outer: for (int i = 0; i < foodCount; i++) {
			HexCell mycell;
			LinkedList<HexCell> path;
			do {
				int tries = 0;
				do {
					mycell = randomCell();
					tries++;
					if (tries > size_x * size_y)
						break outer;
				} while (blocked.contains(mycell));
				tries++;
				if (tries > size_x * size_y)
					break outer;
				path = antHill.aStar(mycell);
			} while (path == null);
			int foodOnCell = randomInRange(1, 5);
			for (int j = 0; j < foodOnCell; j++)
				new Food(mycell);
		}
		/** Útvonalak generálása */
		generatePaths();
		Logger.exiting(this);
	}

	/**
	 * Generáláskor számozott cellák közül visszaadja, hogy az adott sorszámú
	 * szélső cella-e.
	 * 
	 * @param id
	 *            A keresett cella sorszáma
	 * @return Igaz, ha szélső cella
	 */
	public boolean isExitID(int id) {
		Logger.entering(this, new Object[] { id });
		boolean ret = id % size_x == 0 || id % size_x == (size_x - 1) || id < size_x || id >= size_x * (size_y - 1);
		Logger.exiting(this, ret);
		return ret;
	}

	/**
	 * Beállítja a aktív spraynek a következőt.
	 */
	public void nextSpray() {
		int actSprayIndex = sprays.indexOf(activeSpray);
		if (sprays.size() != 0)
			activeSpray = sprays.get((actSprayIndex + 1) % sprays.size());
	}

	/**
	 * Segédfüggvények inicializáláshoz.
	 * 
	 */
	private HexCell randomCell() {
		int cx, cy;
		cx = (int) Math.floor(Math.random() * size_x);
		cy = (int) Math.floor(Math.random() * size_y);
		return getCell(cx, cy);
	}

	/**
	 * Visszaad egy randomszámot a két érték között
	 * 
	 * @param min
	 *            A min érték
	 * @param max
	 *            A max érték
	 * @return Egy random egész szám
	 */
	private int randomInRange(int min, int max) {
		if (max <= min) {
			return max;
		}
		return (int) Math.floor(Math.random() * (max - min) + min);
	}

	/**
	 * Egy {@link ITimed} elem hozzáadása a listához.
	 * 
	 * @param t
	 *            A hozzáadandó elem
	 */
	public void registerTimed(ITimed t) {
		Logger.entering(this, new Object[] { t });
		timedObjects.add(t);
		Logger.exiting(this);
	}

	/**
	 * Beállítja a hangyaboly helyét
	 * 
	 * @param c
	 *            A boly helye
	 */
	public void setAntHillLocation(HexCell c) {
		antHillLocation = c;
	}

	/**
	 * Beállítja a pálya méretét. Előtte letörli a teljes pályát, így
	 * kibővítésre NEM használható. Az első cella a bal felső, és tőle SOUTHEAST
	 * irányban indul el a pálya, minden sor azonos hosszúságú
	 * 
	 * @param x
	 *            Vízszintes méret
	 * @param y
	 *            Függőleges méret
	 */
	public void setSize(int x, int y) {
		Logger.entering(this, new Object[] { x, y });
		/** Méretek beállítása */
		size_x = x;
		size_y = y;
		
		/** Előzőek kitörlése */
		cells.clear();
		foodRemaining = 0;
		antHillLocation = null;
		antCount = 0;
		echidnaCount = 0;

		/** Cellák létrehozása */
		for (int cy = 1; cy <= y; cy++) {
			for (int cx = 1; cx <= x; cx++) {
				HexCell hc = new HexCell(cx, cy);
				hc.setExit(isExitID((cx - 1) + (cy - 1) * size_x));
				cells.add(hc);
			}
		}
		
		/** Szomszédosság beállítása */
		for (int i = 0; i < x * y; i++) {
			HexCell c = cells.get(i);
			if ((i % x) % 2 == 0) {
				if (i - x - 1 >= 0 && i % x != 0)
					c.addCell(Direction.NORTH_WEST, cells.get(i - x - 1));
				if (i - x >= 0)
					c.addCell(Direction.NORTH, cells.get(i - x));
				if (i - x + 1 >= 0 && i % x != (x - 1))
					c.addCell(Direction.NORTH_EAST, cells.get(i - x + 1));
				if (i - 1 >= 0 && i % x != 0)
					c.addCell(Direction.SOUTH_WEST, cells.get(i - 1));
				if (i + x < x * y)
					c.addCell(Direction.SOUTH, cells.get(i + x));
				if (i % x != (x - 1))
					c.addCell(Direction.SOUTH_EAST, cells.get(i + 1));
			} else {
				if (i - 1 > 0 && i % x != 0)
					c.addCell(Direction.NORTH_WEST, cells.get(i - 1));
				if (i - x >= 0)
					c.addCell(Direction.NORTH, cells.get(i - x));
				if (i % x != (x - 1))
					c.addCell(Direction.NORTH_EAST, cells.get(i + 1));
				if (i + x - 1 < x * y && i % x != 0)
					c.addCell(Direction.SOUTH_WEST, cells.get(i + x - 1));
				if (i + x < x * y)
					c.addCell(Direction.SOUTH, cells.get(i + x));
				if (i + x + 1 < x * y && i % x != (x - 1))
					c.addCell(Direction.SOUTH_EAST, cells.get(i + x + 1));
			}
		}
		Logger.exiting(this);
	}

	/**
	 * Aktív spray-ként állítja be a paraméterül kapottat.
	 * 
	 * @param newSpray
	 *            Az új spray
	 */
	public void setSpray(Spray newSpray) {
		Logger.entering(this, new Object[] { newSpray });
		activeSpray = newSpray;
		Logger.exiting(this);
	}

	/**
	 * Az idő múlását szimulálja.
	 * 
	 * Valamilyen sorrendben meghívja az összes beregisztrált {@link ITimed}
	 * objektum {@link ITimed#tick} metódusát. Ha egy tick() hívás hatására egy
	 * új ITimed kerül beregisztrálása, az ő tick metódusa NEM lesz meghívva.
	 * 
	 * Ha a pályán adott mennyiségnél több hangya van, akkor berak néhány
	 * hangyászsünt a pálya széleire.
	 * 
	 * Bizonyos időtartamonként az egy HexCell-en lévő hangyaszagokat
	 * összevonja, hogy ne burjánzanak el az objektumaink.
	 * 
	 * @see ITimed
	 * @see ITimed#tick()
	 */
	public void tick() {
		Logger.entering(this, null);

		/** Szagok összevonása */
		contractionOfScent();

		@SuppressWarnings("unchecked")
		// akiknek még nem hívtuk meg a tickjét
		List<ITimed> unprocessed = (LinkedList<ITimed>) timedObjects.clone();

		/** Sorban mindenkinek meghívjuk a tick()-jét */
		while (unprocessed.size() > 0) {
			unprocessed.get(0).tick();
			unprocessed.remove(0);
		}

		/** Csiga berakás */
		Set<HexCell> usedHexCells = new HashSet<HexCell>();

		if (Math.random() * 100 > 99.0) {
			Random random = new Random();
			HexCell target = cells.get(0);
			int try_number = 0;
			do {
				if (!Main.RND) { // ha nem vélelen akkor a [0,0]-ba rakjuk
					target = getCell(0, 0);
					break;
				}

				// véletlen helyre a széleken
				boolean hor = random.nextBoolean();
				boolean ver = random.nextBoolean();
				if (ver) {
					target = getCell(hor ? 0 : size_x - 1, random.nextInt(size_y));
				} else {
					target = getCell(random.nextInt(size_x), ver ? 0 : size_y - 1);
				}
				usedHexCells.add(target);
				++try_number;
				// megpróbáljuk oda tenni, ahová még nem tettünk (de nem
				// erőlködünk vele)
			} while (usedHexCells.contains(target) && try_number < 2 * (size_x + size_y));

			new Snail(target);
		}

		/** Hangyászsünök berakása */
		for (int i = (antCount / echidnaAttractingAntQuantity) - echidnaCount; i > 0; i--) {
			Random random = new Random();
			HexCell target = cells.get(0);
			int try_number = 0;
			do {
				if (!Main.RND) { // ha nem vélelen akkor a [0,0]-ba rakjuk
					target = getCell(0, 0);
					break;
				}

				// véletlen helyre a széleken
				boolean hor = random.nextBoolean();
				boolean ver = random.nextBoolean();
				if (ver) {
					target = getCell(hor ? 0 : size_x - 1, random.nextInt(size_y));
				} else {
					target = getCell(random.nextInt(size_x), ver ? 0 : size_y - 1);
				}
				usedHexCells.add(target);
				++try_number;
				// megpróbáljuk oda tenni, ahová még nem tettünk (de nem
				// erőlködünk vele)
			} while (usedHexCells.contains(target) && try_number < 2 * (size_x + size_y));

			new Echidna(target);
		}

		/** Ha elfogyott az összes étel, szól */
		if (foodRemaining == 0) {
			Controller.getInstance().runOutOfFood();
		}
		Logger.exiting(this);
	}
}
