package main;

import main.Visualizer.Layer;

/**
 * 
 * Ételeket reprezentáló osztály
 * 
 */
public class Food extends NonBlocking {

	/**
	 * Hangyák befolyásolásának mértéke.
	 * 
	 * @see Ant#move(HexCell)
	 */
	private static int influence = 500;
	/** A kirajzoláshoz a képe */
	private int type;
	/** A hozzá tartozó minikép */
	DummyImage smallImage;
	
	/**
	 * Alapértelmezett konstruktor
	 * 
	 * @param location
	 *            Pozíció
	 */
	public Food(HexCell location) {
		super(location);
		Logger.entering(this, new Object[] { location });
		
		int rnd = (int) Math.floor(Math.random() * 3);
		type = rnd;
		smallImage = new DummyImage(null, drawCarried(), Layer.OVERLAY);
		if (location != null) {
			new FoodScent(location, this, 3, 2);
			Director.getInstance().foodGenerated();
		}
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void accept(IInterActor interactor) {
		Logger.entering(this, new Object[] { interactor });
		interactor.interactWith(this);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public String draw() {
		return "img/food0"+type+".png";
	}

	/**
	 * Visszaadja az elhordás közbeni képét az ételnek.
	 */
	public String drawCarried() {
		return "img/food0"+type+"_p.png";
	}

	/**
	 * Ételt az adott mezőre ejtik ezzel.
	 * 
	 * @param target
	 *            A következő pozíció
	 */
	public void drop(HexCell target) {
		Logger.entering(this, new Object[] { target });
		location = target;
		location.addObject(this);
		smallImage.removeFromCell();
		new FoodScent(location, this, 3, 2);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public Layer getLayer() {
		return Layer.OBSTACLE;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int influence() {
		return influence;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "food";
	}
}