package main;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.List;

import main.HexCell.Direction;

/**
 * Betötéshez és mentéshez használt osztály
 *
 */
public class LoaderSaver {

	/**
	 * A későbbiekben használatra kerülő spray-k időzítéséhez használt belső
	 * osztály
	 * 
	 * @author Kozaroczy
	 * 
	 */
	static class doSpraying {
		int tick;
		Spray spray;
		HexCell location;

		/** Konstruktor */
		public doSpraying(HexCell loc, int tick, Spray spray) {
			location = loc;
			this.tick = tick;
			this.spray = spray;
		}
	}

	/**
	 * Ebben adja vissza, hogy meddig kell futtatni a beolvasott adatokból
	 * 
	 */
	public static class RunCondition {
		int ticks;
		String functionName;
		String returnValue;

		RunCondition() {
			ticks = 0;
			functionName = null;
			returnValue = null;
		}
	}

	/**
	 * Kiírja a pályát a prototípus koncepciója 7.1.2-es pontjának megfelelően.
	 * A függvény meghívásához a {@link Director}t inicializálni kell.
	 * 
	 * @param out
	 *            Ahová írni akarjuk a pályát. (A Streamet nem zárja be.)
	 * @see Located#toString()
	 */
	public static void exportMap(PrintStream out) {
		Director d = Director.getInstance();
		List<HexCell> cells = d.getCells();

		out.println("width " + d.size_x);
		out.println("height " + d.size_y);

		for (int x = 0; x < d.size_x; ++x) {
			for (int y = 0; y < d.size_y; ++y) {
				// a cellán lévő mindenkit kiírunk
				HexCell cell = cells.get(d.size_x * y + x);
				for (Located located : cell.getAll()) {
					StringBuilder sb = new StringBuilder("add ");
					sb.append(x + 1);
					sb.append(' ');
					sb.append(y + 1);
					sb.append(' ');
					sb.append(located);
					out.println(sb);
				}
			}
		}

	}

	/**
	 * A pályakép beolvasása. Egy hatalmas állapotgéppel történik.
	 * 
	 * @param in
	 *            A beolvasandó forrás
	 * @return A lefuttatandó tick-ek száma
	 */
	public static RunCondition importMap(InputStream in) {
		Director director = Director.getInstance();
		BufferedReader br = new BufferedReader(new InputStreamReader(in));
		String line = null;
		RunCondition condition = new RunCondition();
		boolean generated = false;
		boolean hasAntHill = false;
		try {
			/** Amíg van mit beolvasni */
			while ((br.ready())) {
				line = br.readLine();
				String[] parts = line.split(" ");
				if (parts[0].equals("width")) {
					/** A szélesség megadása */
					director.setSize(Integer.parseInt(parts[1]), Director.getInstance().size_y);
				} else if (parts[0].equals("height")) {
					/** A magasság megadása */
					director.setSize(Director.getInstance().size_x, Integer.parseInt(parts[1]));
				} else if (parts[0].equals("generate")) {
					/** Automatikus generálás */
					director.initialize();
					generated = true;
				} else if (parts[0].equals("add")) {
					/** Located hozzáadása */
					
					/** Először mindig a helyzetet kell megadni */
					int x = Integer.parseInt(parts[1]);
					int y = Integer.parseInt(parts[2]);

					HexCell location = director.getCell(x - 1, y - 1);

					Direction dir;
					int speed;
					int size;
					int remaining;
					
					/** Ez a paraméter mondja meg, hogy mit adunk hozzá */
					switch (parts[3]) {
					case "ant":
						/** Hangya hozzáadása */
						/** Ha tudjuk az irányt, beállítjuk */
						dir = parts.length >= 5 ? Direction.fromString(parts[4]) : Direction.NORTH;
						/** Ha van nála kaja hozzáadunk */
						boolean hasFood = parts.length >= 6 ? parts[5].equals("1") : false;
						/** Beállítjuk a sebességét */
						speed = parts.length >= 7 ? Integer.parseInt(parts[6]) : 2;
						/** Majd létrehozzuk */
						new Ant(location, dir, speed, hasFood);
						break;
					case "anthill":
						/** Hangyaboly, létrehozzuk */
						new AntHill(location);
						hasAntHill = true;
						break;
					case "antlion":
						/** Hangyaleső, létrehozzuk */
						new Antlion(location);
					case "antscent":
						remaining = parts.length >= 5 ? Integer.parseInt(parts[4]) : 15;
						new AntScent(location, remaining);
						break;
					case "antscentvanisher":
						size = parts.length >= 5 ? Integer.parseInt(parts[4]) : 15;
						new AntScentVanisherScent(location, size);
						break;
					case "echidna":
						dir = parts.length >= 5 ? Direction.fromString(parts[4]) : Direction.NORTH;
						int antsEaten = parts.length >= 6 ? Integer.parseInt(parts[5]) : 0;
						speed = parts.length >= 7 ? Integer.parseInt(parts[6]) : 4;
						new Echidna(location, dir, speed, antsEaten);
						break;
					case "food":
						new Food(location);
						break;
					case "foodscent":
						// ilyet nem olvasunk be
						break;
					case "pebble":
						new Pebble(location);
						break;
					case "poison":
						size = parts.length >= 5 ? Integer.parseInt(parts[4]) : 4;
						remaining = parts.length >= 6 ? Integer.parseInt(parts[5]) : 15;
						new Poison(location, size, remaining);
						break;
					case "puddle":
						new Puddle(location);
						break;
					default:
						break;
					}
				} else if (parts[0].equals("do")) {
					/** Késleltetett sprayfújás létrehozása */
					int x = Integer.parseInt(parts[1]);
					int y = Integer.parseInt(parts[2]);
					int tick = Integer.parseInt(parts[3]);
					HexCell cell = director.getCell(x - 1, y - 1);

					switch (parts[4]) {
					case "antscentvanisher":
						director.sprayingList.add(new doSpraying(cell, tick, new AntScentVanisherSpray()));
						break;
					case "poison":
						director.sprayingList.add(new doSpraying(cell, tick, new PoisonSpray()));
						break;
					}
				} else if (parts[0].equals("run")) {
					/** Futtatás adott tick()-ig */
					int newTicks = parts.length >= 2 ? Integer.parseInt(parts[1]) : 0;
					condition.ticks = newTicks;
				} else if (parts[0].equals("until")) {
					/** Futtatás adott tick()-ig vagy egy feltétel teljesüléséig */
					if (parts.length > 1) {
						int maxTicks = 100;
						String returnValue = null;
						if (parts.length > 2) {
							maxTicks = Integer.parseInt(parts[2]);
							if (parts.length > 3) {
								if (parts[3].equals("#null")) {
									returnValue = null;
								} else {
									returnValue = parts[3];
								}
							}
						}
						condition.ticks = maxTicks;
						condition.functionName = parts[1];
						condition.returnValue = returnValue;
					}
				}
			}
		} catch (Exception e) {
			System.err.println("Hiba a fájlból olvasáskor");
		} finally {
			try {
				br.close();
			} catch (IOException e) {
				System.err.println(br.toString() + " could not be closed.");
			}
		}
		/** Ha nincs útvonal generálva, de van hangyaboly, akkor generáljunk */
		if (!generated && hasAntHill)
			director.generatePaths();
		/** Ha nem lett tick-szám megadva, legyen 15 tick */
		condition.ticks = condition.ticks != 0 ? condition.ticks : 15;
		return condition;
	}
}
