package main;

import main.Visualizer.Layer;

/**
 * A közös ősosztálya mindennek, aminek van pozíciója
 * 
 */
public abstract class Located implements IAccepter {

	/** Az objektum pozíciója */
	protected HexCell location;

	/**
	 * Az alapértelmezett konstruktort felüldefiniálja, így csak akkor hozható
	 * létre, ha megadjuk a helyét. Hozzáadja magát az adott cellához, ha az nem
	 * null
	 * 
	 * @param location
	 *            Alapértelemezett pozíció
	 */
	public Located(HexCell location) {
		// Logger.entering(this, new Object[] { location });
		this.location = location;
		if (location != null) {
			location.addObject(this);
		}
		// Logger.exiting(this);
	}

	/**
	 * Eltávolítja magát a cellából
	 * 
	 */
	public void dispose() {
		Logger.entering(this, null);
		try {
			location.removeObject(this);
		} catch (NullPointerException e) {
			// Nothing to do here
		}
		Logger.exiting(this);
	}

	/**
	 * Kirajzoláshoz használt függvény
	 * 
	 * @return A használandó fájl neve
	 */
	public abstract String draw();

	/**
	 * Finalize, amit a garbage collector használ
	 * 
	 */
	public void finalize() throws Throwable {
	}

	/** Visszaadja, hogy kirajzoláskor melyik rétegben kell lennie */
	public abstract Layer getLayer();

	/**
	 * Megadja, hogy mennyire befolyásolja a hangya útvonalát.
	 */
	public int influence() {
		Logger.entering(this, null);
		Logger.exiting(this, 0);
		return 0;
	}

	/** Pályakép kiírásához felüldeifiniálja a toString() függvényt. */
	public abstract String toString();

}