package main;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.HashSet;

/**
 * Ha nem null értékkel inicializáljuk, akkor oda logolja a függvényhívásokat.
 * Mindene statikus, hogy ne kelljen vesződni a paraméterként való átadással.
 * 
 * @author pgy
 * @author Kozaroczy
 */
public class Logger {

	/** A kimenet */
	protected static PrintStream out;
	/** A behúzás mértéke */
	private static int indentLevel = -1;
	/** Két space */
	private static String indent = "  ";
	/**
	 * Függvényhívások és visszatérési értékek logja, az until tesztbemenet
	 * megvalósításához.
	 */
	private static HashMap<String, HashSet<String>> calls = new HashMap<String, HashSet<String>>();
	/**
	 * Éppen használatban van-e a logolás
	 * 
	 * @see #begin()
	 * @see #end()
	 */
	private static boolean isLogging = false;

	/**
	 * Elindítja a logolást. Lekapcsolni az {@link #end()}-el lehet.
	 */
	public static void begin() {
		calls.clear();
		if (out != null)
			isLogging = true;
	}

	/**
	 * @param function
	 *            Keresett függvény
	 * @param returnValue
	 *            Várt visszatérési érték
	 * @return Igazzal tér vissza, ha meghívódott a keresett függvény a várt
	 *         visszatérési értékkel
	 */
	public static boolean called(String function, String returnValue) {
		if (calls.containsKey(function)) {
			if (returnValue != null) {
				HashSet<String> values = calls.get(function);
				return values.contains(returnValue);
			} else {
				return true;
			}
		} else {
			return false;
		}
	}

	/**
	 * Logfájl erőforrás felszabadítása. Szabályosan bezárja a logfájlt.
	 */
	public static void dispose() {
		if (isLogging) {
			Logger.end();
		}
		out.close();
		out = null;
	}

	/**
	 * Leállítja a logolást. Elindítani a {@link #begin()}-el lehet.
	 */
	public static void end() {
		calls.clear();
			isLogging = false;
	}

	/**
	 * Függvényhívást logoló metódus.
	 * 
	 * @param that
	 *            A hívó metódus referenciája.
	 * @param params
	 *            A logolandó paraméterek tömbje.
	 */
	public static void entering(Object that, Object[] params) {
		if (!isLogging)
			return;

		indentLevel++;

		StringBuilder s = new StringBuilder("");
		for (int i = 0; i < indentLevel; ++i) {
			s.append(indent);
		}
		s.append(Thread.currentThread().getStackTrace()[2].getClassName());
		s.append('=');
		s.append(that != null ? that.hashCode() : "null");
		s.append("::");
		s.append(Thread.currentThread().getStackTrace()[2].getMethodName());
		s.append('(');
		if (params != null) {
			for (int i = 0; i < params.length; ++i) {
				if (params[i] == null) {
					s.append("null");
				} else {
					s.append(params[i].getClass().getName());
					s.append('=');
					s.append(params[i].hashCode());
				}
				if (i != params.length - 1)
					s.append(", ");
			}
		}
		s.append(')');

		out.println(s);
	}

	/**
	 * Egyparaméteres {@link #exiting(Object, Object)}, visszatérési értéket nem
	 * logol.
	 * 
	 * @param that
	 *            A hívó objektum.
	 */
	public static void exiting(Object that) {
		if (!isLogging)
			return;

		String methodName = Thread.currentThread().getStackTrace()[2].getClassName().replaceAll("main\\.", "") + "::" + Thread.currentThread().getStackTrace()[2].getMethodName();
		if (calls.containsKey(methodName)) {
			calls.get(methodName).add("#null");
		} else {
			HashSet<String> temp = new HashSet<String>();
			temp.add("#null");
			calls.put(methodName, temp);
		}
		StringBuilder s = new StringBuilder("");
		for (int i = 0; i <= indentLevel; ++i) {
			s.append(indent);
		}
		s.append("ret ");
		s.append(Thread.currentThread().getStackTrace()[2].getClassName());
		s.append('=');
		s.append(that != null ? that.hashCode() : "null");
		s.append("::");
		s.append(Thread.currentThread().getStackTrace()[2].getMethodName());
		indentLevel--;
		out.println(s);
	}

	/**
	 * Függvényvisszatérést logoló metódus. Kiírja, hogy melyik osztály melyik
	 * metódusából hívták, illetve megadható, hogy a hívó metódus mivel fog
	 * visszatérni.
	 * 
	 * @param that
	 *            A hívó objektum.
	 * @param returnValue
	 *            A hívó függvény visszatérési értéke.
	 */
	public static void exiting(Object that, Object returnValue) {
		if (!isLogging)
			return;

		String methodName = Thread.currentThread().getStackTrace()[2].getClassName().replaceAll("main\\.", "") + "::" + Thread.currentThread().getStackTrace()[2].getMethodName();
		if (calls.containsKey(methodName)) {
			calls.get(methodName).add(returnValue != null ? returnValue.toString() : "#null");
		} else {
			HashSet<String> temp = new HashSet<String>();
			temp.add(returnValue != null ? returnValue.toString() : "#null");
			calls.put(methodName, temp);
		}
		StringBuilder s = new StringBuilder("");
		for (int i = 0; i <= indentLevel; ++i) {
			s.append(indent);
		}
		s.append("ret ");
		s.append(Thread.currentThread().getStackTrace()[2].getClassName());
		s.append('=');
		s.append(that != null ? that.hashCode() : "null");
		s.append("::");
		s.append(Thread.currentThread().getStackTrace()[2].getMethodName());
		s.append("() with: ");
		s.append(returnValue != null ? returnValue.getClass().getName() : "null");
		s.append('=');
		s.append(returnValue != null ? returnValue.hashCode() : "null");

		indentLevel--;
		out.println(s);
	}

	/** Default init(). */
	public static void init() {
		init(null);
	}

	/**
	 * Inicializálja a Loggert. A Logger metódusai közül ezt kell először
	 * elindítani.
	 * 
	 * A naplózást nem indítja el automatikusan. A logolást a
	 * {@link main.Logger#begin()} függvénnyel lehet elindítani.
	 * 
	 * @param destination
	 *            A logolás célpontja. Ha null, akkor automatikusan a
	 *            skeleton.log fájlba ment.
	 */
	public static void init(PrintStream destination) {
		if (destination != null) {
			out = destination;
		}

	}

	/**
	 * Logolja a paraméterként kapott szöveget a megfelelő behúzással.
	 * 
	 * @param str
	 *            A beszúrandó megjegyzés.
	 */
	public static void meanwhiling(String str) {
		if (!isLogging)
			return;
		StringBuilder s = new StringBuilder("");
		for (int i = 0; i < indentLevel + 1; ++i) {
			s.append(indent);
		}
		s.append(str);
		out.println(s);
	}

	/**
	 * Alaphelyzetbe állítja a Logger függvényhívás-cachét.
	 */
	public static void reset() {
		calls.clear();
	}

}
