package main;

import java.util.ArrayList;

import main.HexCell.Direction;
import main.Visualizer.Layer;

/**
 * 
 * Kavicsokat reprezentáló osztály
 * 
 */
public class Pebble extends Barrier implements IInterActor {

	/** Ha meglökik a kavicsot, ez határozza meg az irányát */
	Direction direction;

	/** A kirajzoláskor melyik képet használja */
	int type;
	
	/** Alapértelmezett konstruktor */
	public Pebble(HexCell location) {
		super(location);
		int rnd = (int) Math.floor(Math.random() * 3);
		type = rnd;
		Logger.entering(this, new Object[] { location });
		direction = Direction.NORTH;
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void accept(IInterActor interactor) {
		Logger.entering(this, new Object[] { interactor });
		if (interactor != this) {
			interactor.interactWith(this);
		}
		Logger.exiting(this);

	}

	/**
	 * {@inheritDoc}
	 */
	public String draw() {
		return "img/pebble"+type+".png";
	}

	/**
	 * {@inheritDoc}
	 */
	public Layer getLayer() {
		return Layer.OBSTACLE;
	}

	/**
	 * Hangyával való interakció. Megöli a hangyát
	 * 
	 * @see Ant#die()
	 */
	public void interactWith(Ant a) {
		Logger.entering(this, new Object[] { a });
		a.die();
		Logger.exiting(this);
	}

	/**
	 * Hangyabollyal való interakció. Visszalöki a követ.
	 */
	public void interactWith(AntHill a) {
		Logger.entering(this, new Object[] { a });
		push(direction.back());
		Logger.exiting(this, null);
	}

	/**
	 * Hangyalesővel való interakció. Elpusztul a hangyaleső.
	 */
	public void interactWith(Antlion a) {
		Logger.entering(this, new Object[] { a });
		a.dispose();
		Logger.exiting(this, null);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(AntScent a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void interactWith(AntScentVanisherScent a) {
		Logger.entering(this, new Object[] { a });
		// Nothing to do here
		Logger.exiting(this);
	}

	/**
	 * Ahová belép, onnan visszalöki a hangyászsünt.
	 */
	public void interactWith(Echidna e) {
		Logger.entering(this, new Object[] { e });
		e.move(e.location.getCell(e.direction.back()));
		Logger.exiting(this);
	}

	/**
	 * Étellel való interakció. Elpusztul az étel.
	 * 
	 * @see Director#foodDestroyed()
	 */
	public void interactWith(Food f) {
		Logger.entering(this, new Object[] { f });
		f.dispose();
		Director.getInstance().foodDestroyed();
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(FoodScent s) {
		Logger.entering(this, new Object[] { s });
		Logger.exiting(this);
	}

	/**
	 * Ha már van ott kavics, ahová lép, onnan visszalép.
	 * 
	 */
	public void interactWith(Pebble p) {
		Logger.entering(this, new Object[] { p });
		push(direction.back());
		Logger.exiting(this);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Poison p) {
		Logger.entering(this, new Object[] { p });
		Logger.exiting(this);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Puddle p) {
		Logger.entering(this, new Object[] { p });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Slime s) {
		Logger.entering(this, new Object[] { s });
		s.dispose();
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Snail s) {
		Logger.entering(this, new Object[] { s });
		s.move(location.getCell(direction.back()));
		Logger.exiting(this);
	}

	/**
	 * Átmozog a megfelelő irányba
	 * 
	 * @param dir
	 *            A mozgás iránya
	 */
	public void push(Direction dir) {
		Logger.entering(this, new Object[] { dir });

		direction = dir;

		HexCell c = location.getCell(dir);
		if (c != null) {
			location.removeObject(this);
			c.addObject(this);
			location = c;
			ArrayList<Located> othersOnNext = location.getAll();
			for (int i = othersOnNext.size() - 1; i >= 0; i--) {
				othersOnNext.get(i).accept(this);
			}
		} else {
			dispose();
		}
		Logger.exiting(this);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "pebble";
	}
}