package main;

import java.util.HashMap;

import main.HexCell.Direction;
import main.Visualizer.Layer;

/**
 * 
 * Mérgeket reprezentáló közös ősosztály
 * 
 */
public class Poison extends Scent {

	/**
	 * Alapértelmezett konstruktor
	 * 
	 * @param location
	 *            Pozíció
	 * @param size
	 *            Terjedés mérete
	 */
	public Poison(HexCell location, int size) {
		super(location, size, 4);
		Logger.entering(this, new Object[] { location, size });
		Logger.exiting(this);
	}

	/**
	 * Betöltéshez használt konstruktor
	 * 
	 * @param location
	 *            Pozíció
	 * @param size
	 *            Terjedés mérete
	 * @param remaining
	 *            A szag intenzitása
	 */
	public Poison(HexCell location, int size, int remaining) {
		super(location, size, remaining);
		Logger.entering(this, new Object[] { location, size, remaining });
		Logger.exiting(this);
	}

	/**
	 * Terjedéshez használt konstruktor
	 * 
	 * @param location
	 *            Pozíció
	 * @param size
	 *            Terjedés mérete
	 * @param remaining
	 *            Hátralévő intenzitás
	 * @param dir
	 *            A terjedés iránya
	 */
	public Poison(HexCell location, int size, int remaining, Direction dir) {
		super(location, size, remaining, dir);
		Logger.entering(this, new Object[] { location, size, remaining, dir });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void accept(IInterActor interactor) {
		Logger.entering(this, new Object[] { interactor });
		interactor.interactWith(this);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public String draw() {
		return "img/poison.png";
	}

	/**
	 * {@inheritDoc}
	 */
	public Layer getLayer() {
		return Layer.SCENT;
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void tick() {
		Logger.entering(this, null);
		/** Ha elfogyott az ideje, elillan */
		if (remaining == 0) {
			dispose();
		}
		
		if (radius > 0) {
			/** Ha még tud terjedni */
			HashMap<Direction, HexCell> adjacent = location.getAdjacents();
			if (first) {
				/** Ha ő az első, akkor mindenirányba terjed */
				for (Direction d : Direction.values()) {
					HexCell c = adjacent.get(d);
					if (c != null)
						new Poison(c, radius - 1, 4, d);
				}
			} else {
				/** Egyébként csak a meghatározott irányokba */
				for (int i = 0; i < dir.size(); i++) {
					HexCell c = adjacent.get(dir.get(i));
					if (c != null) {
						new Poison(c, radius - 1, remaining, dir.get(i));
					}
				}
			}
			/** Ha terjedt, ne terjedjen többet */
			radius = 0;
		}
		remaining--;
		Logger.exiting(this);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "poison " + radius + " " + this.remaining;
	}
}
