package main;

import java.util.ArrayList;

import main.HexCell.Direction;

/**
 * Szagok közös ősosztálya
 *
 */
public abstract class Scent extends NonBlocking implements ITimed {

	/** Az elillanásig hátralévő idő */
	protected int remaining;
	/** A terjedés nagysága */
	protected int radius;
	/** A terjedés iránya. Akkor használja, ha nem ő az első szagfolt */
	protected ArrayList<Direction> dir;
	/** Igaz, ha ő az első szagfolt. Fontos a terjedéshez */
	protected boolean first;

	/**
	 * Alapértelmezett konstruktor
	 * 
	 * @param location
	 *            Pozíció
	 * @param size
	 *            A terjedés mértéke
	 * @param remaining
	 *            A maradék idő
	 */
	public Scent(HexCell location, int size, int remaining) {
		super(location);
		Logger.entering(this, new Object[] { location });
		Director.getInstance().registerTimed(this);
		this.remaining = remaining;
		radius = size;
		first = true;
		dir = new ArrayList<Direction>();
		for (Direction d : Direction.values())
			dir.add(d);
		Logger.exiting(this);
	}

	/**
	 * Terjedéshez használt konstruktor
	 * 
	 * @param location
	 *            Pozíció
	 * @param size
	 *            Terjedés mérete
	 * @param remaining
	 *            Elillanásig hétralévő idő
	 * @param originDir
	 *            Terjedés iránya
	 */
	public Scent(HexCell location, int size, int remaining, Direction originDir) {
		super(location);
		Logger.entering(this, new Object[] { location });
		Director.getInstance().registerTimed(this);
		this.remaining = remaining;
		radius = size;
		first = false;
		dir = new ArrayList<Direction>();
		dir.add(originDir);
		dir.add(originDir.next());
		Logger.exiting(this);
	}

	/**
	 * Új terjedési irányok hozzáadása
	 * 
	 * @param newDirs Lista, amit hozzáfűz az eddigiekhez
	 */
	public void addDirs(ArrayList<Direction> newDirs) {
		Logger.entering(this, new Object[] { newDirs });
		for (Direction d : newDirs) {
			boolean contains = false;
			for (Direction o : dir) {
				if (o == d) {
					contains = true;
					break;
				}
			}
			if (!contains) {
				dir.add(d);
			}
		}
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void dispose() {
		Logger.entering(this, null);
		Director.getInstance().deregisterTimed(this);
		dir.clear();
		super.dispose();
		Logger.exiting(this);
	}

	/**
	 * Visszaadja az elillanásig hátralévő időt
	 * 
	 * @return A maradék idő
	 */
	public int remaining() {
		return remaining;
	}

	/**
	 * {@inheritDoc} Minden meghíváskor ha tud még, akkor tovább terjed a
	 * megfelelő irányokba, egyébként pedig folyamatosan csökken az intenzitása.
	 */
	public abstract void tick();
}
