package main;

import main.HexCell.Direction;
import main.Visualizer.Layer;

/**
 * 
 * Csigákat reprezentáló osztály
 * 
 */
public class Snail extends Wanderer {
	/** Az alapértelmezett sebesség, értéke {@value} */
	protected static final int defaultSpeed = 3;
	/** A hagyott nyálka élettartama */
	protected static final int slimeLife = 15;
	/** Minimim {@value} tick-et tölt a pályán */
	protected static final int minimumOn = 30;
	/** Ennyi ideje van a pályán */
	protected int on;
	/** Hamis, ha nem kell nyálkát hagynia */
	protected boolean putsSlime;

	/**
	 * Alapértelmezett konstruktor
	 * 
	 * @param location
	 *            A csiga pozíciója
	 */
	Snail(HexCell location) {
		super(location);
		Logger.entering(this, new Object[] { location });
		speed = defaultSpeed;
		on = 0;
		putsSlime = true;
		Logger.exiting(this);
	}

	/**
	 * A hangyászsün betöltéshez használt konstruktora
	 * 
	 * @param location
	 *            Pozíció
	 * @param dir
	 *            Irány
	 * @param speed
	 *            Sebesség
	 */
	Snail(HexCell location, Direction dir, int speed) {
		super(location, dir, speed);
		Logger.entering(this, new Object[] { location, dir, speed });
		on = 0;
		putsSlime = true;
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void accept(IInterActor interactor) {
		Logger.entering(this, new Object[] { interactor });
		interactor.interactWith(this);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public String draw() {
		return "img/snail.png";
	}

	/**
	 * {@inheritDoc}
	 */
	public Layer getLayer() {
		return Layer.ACTOR;
	}

	/**
	 * {@inheritDoc} A hangya nem lép vele egy mezőre
	 */
	public void interactWith(Ant a) {
		Logger.entering(this, new Object[] { a });
		a.move(location.getCell(a.direction.back()));
		Logger.exiting(this);
	}

	/**
	 * Nem megy rá a tócsára
	 */
	public void interactWith(Puddle p) {
		Logger.entering(this, new Object[] { p });
		move(location.getCell(direction.back()));
		Logger.exiting(this);
	}

	/**
	 * Nyálkával való interakció. Felfrissíti az elillanásig maradó idejét.
	 */
	public void interactWith(Slime s) {
		Logger.entering(this, new Object[] { s });
		s.remaining = slimeLife;
		putsSlime = false;
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Snail s) {
		Logger.entering(this, new Object[] { s });
		if (this.location == s.location && s != this) {
			move(location.getCell(direction.back()));
			direction = direction.next();
		}
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}. {@link Slime}-ot hagy maga után
	 */
	public void move(HexCell target) {
		Logger.entering(this, new Object[] { target });
		if (target == null) {
			return;
		}
		speed = defaultSpeed;
		super.move(target);
		if (putsSlime) {
			new Slime(this.location, slimeLife);
		}
		Logger.exiting(this, new Object[] { target });
	}

	/**
	 * {@inheritDoc} Véletlenszerű irányba mozog.
	 */
	public void tick() {
		Logger.entering(this, null);
		putsSlime = true;
		unusedTicks++;
		if (unusedTicks >= speed) {
			if (Main.RND) {
				int rnd = (int) (Math.random() * 4);
				if (rnd == 0)
					direction = direction.prev();
				else if (rnd == 3)
					direction = direction.next();
			}
			HexCell new_loc = location.getCell(direction);
			if (new_loc != null) {
				move(new_loc);
			} else {
				if (on < minimumOn) {
					direction = direction.next();
				} else {
					dispose();
				}
			}
			unusedTicks = 0;
		}
		on++;
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "snail " + this.direction + " " + this.speed;
	}
}
