package main;

import java.util.ArrayList;

import main.HexCell;
import main.HexCell.Direction;

/**
 * A mozogni képes lények közös ősosztálya
 *
 */
public abstract class Wanderer extends Located implements ITimed, IInterActor {

	/** A mozgás iránya */
	Direction direction;
	/** A mozgás sebessége */
	int speed;
	/** Az előző mozgás óta eltelt tick-ek száma */
	protected int unusedTicks;

	/**
	 * Kóricálók konstruktora.
	 * 
	 * @param location
	 *            Alapértelemezett pozíció
	 * 
	 */
	Wanderer(HexCell location) {
		super(location);
		Logger.entering(this, new Object[] { location });
		direction = Direction.NORTH;
		if (Main.RND) {
			int rnd = (int) (Math.random() * Direction.values().length);
			for (int j = 0; j < rnd; j++) {
				direction = direction.next();
			}
		}
		Director.getInstance().registerTimed(this);
		unusedTicks = 0;
		Logger.exiting(this);
	}

	/**
	 * Minden paraméterre kiterjedő konstruktor pályabetöltéshez
	 * 
	 * @param location
	 *            Pozíció
	 * @param dir
	 *            Irány
	 * @param speed
	 *            Sebesség
	 */
	Wanderer(HexCell location, Direction dir, int speed) {
		super(location);
		Logger.entering(this, new Object[] { location, dir, speed });
		direction = dir;
		this.speed = speed;
		Director.getInstance().registerTimed(this);
		unusedTicks = 0;
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public void dispose() {
		Logger.entering(this, null);
		Director.getInstance().deregisterTimed(this);
		super.dispose();
		Logger.exiting(this);
	}

	/**
	 * A sebességet adja vissza.
	 * 
	 * @return Sebesség
	 * 
	 */
	public int getSpeed() {
		Logger.entering(this, null);
		Logger.exiting(this, speed);
		return speed;
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Ant a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(AntHill a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Antlion a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(AntScent a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(AntScentVanisherScent a) {
		Logger.entering(this, new Object[] { a });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Echidna e) {
		Logger.entering(this, new Object[] { e });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Food f) {
		Logger.entering(this, new Object[] { f });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(FoodScent s) {
		Logger.entering(this, new Object[] { s });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc} Visszapattan a kavicsról.
	 */
	public void interactWith(Pebble p) {
		Logger.entering(this, new Object[] { p });
		if (this.location == p.location) {
			move(location.getCell(direction.back()));
			direction = direction.next();
		}
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void interactWith(Poison p) {
		Logger.entering(this, new Object[] { p });
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc} A sebességét a felére csökkenti
	 */
	public void interactWith(Puddle p) {
		Logger.entering(this, new Object[] { p });
		setSpeed(speed * 2);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc} A sebességét a felére csökkenti
	 */
	public void interactWith(Slime s) {
		Logger.entering(this, new Object[] { s });
		setSpeed(speed * 2);
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc} Visszapattan a csigáról.
	 */
	public void interactWith(Snail s) {
		Logger.entering(this, new Object[] { s });
		if (this.location == s.location) {
			move(location.getCell(direction.back()));
			direction = direction.next();
		}
		Logger.exiting(this);
	}

	/**
	 * Mozgás a kijelölt cellába, majd minden ottlévőtől interakciót vár
	 * 
	 * @param next
	 *            A következő cella
	 */
	public void move(HexCell next) {
		Logger.entering(this, new Object[] { next });
		
		if (next == null) {
			direction = direction.next();
			Logger.meanwhiling("Wanderer move() nem kapott irányt");
			Logger.exiting(this);
			return;
		}

		location.removeObject(this);
		next.addObject(this);
		location = next;

		ArrayList<Located> othersOnNext = location.getAll();
		for (int i = othersOnNext.size() - 1; i >= 0; i--) {
			othersOnNext.get(i).accept(this);
		}

		Logger.exiting(this);
	}

	/**
	 * A sebességet állítja be.
	 * 
	 * @param newVal
	 *            Az új sebesség
	 */
	public void setSpeed(int newVal) {
		Logger.entering(this, new Object[] { newVal });
		speed = newVal;
		Logger.exiting(this);
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	public abstract void tick();

}
